// filename - SignalHandling.h

#ifndef SIGNALHANDLING_H
#define SIGNALHANDLING_H

#include <csignal>
#include <stdio.h>
#include <sys/fcntl.h>
#include <unistd.h>

struct SignalHandling
{
   void operator() (int signal, void performSigAction (int s), int fd = -1)
   {
      struct sigaction sigParam;
      if (sigfillset(&sigParam.sa_mask) < 0)
      {
         perror("SignalHandling::PrepareSIGIO cannot block other signals");
      }
      sigParam.sa_flags = 0;
      sigParam.sa_handler = performSigAction;
      if (sigaction(signal, &sigParam, NULL) < 0)
      {
         perror("SignalHandling::PrepareSIGIO unable to change default behavior after signal delivery");
      }

      if (signal == SIGIO)
      {
         const auto CURRENT_PROCESS = getpid();
         if (fcntl(fd, F_SETOWN, CURRENT_PROCESS) < 0)
         {
            perror("SignalHandling::PrepareSIGIO unable to set ownership of descriptor's SIGIO to current process");
         }
         if (fcntl(fd, F_SETFL, O_ASYNC) < 0)
         {
            perror("SignalHandling::PrepareSIGIO unable to set file descriptor for asynchronous delivery of signals");
         }
      } 
   }
};

SignalHandling RegisterHandlerForSignal;
#endif
