// filename - SocketHandler.h

#ifndef SOCKETHANDLER_H
#define SOCKETHANDLER_H

#include <functional>
#include <string>
#include <sys/types.h>
#include <netinet/in.h>
#include <unistd.h>

#define PORT_ONE 51001
#define PORT_TWO 51002

class SocketHandler
{
   enum InitializationStatus
   {
      Uninitialized,
      Initialized
   };
   const std::size_t PACKET_SIZE = 21845;
   InitializationStatus init_status;
   int                  socket_fd; // socket file descriptor
   int                  connected_socket_fd;
   int                  port_no;
   struct sockaddr_in   connect_to_addr;
   bool                 is_client_connected = false;

public:
   SocketHandler (int socket_type, int port);
   ~SocketHandler ();

   int InitializeAsServer ();
   void ConnectToServer (std::string ip_address);
   void ConnectToClient ();
   int GetFileDescriptor ();
   ssize_t Send (const void * data, std::size_t len);
   ssize_t Receive (void * buffer, std::size_t len = 1);
   ssize_t SendAndWait (const void * data, std::size_t len);
   ssize_t ReceiveAndAcknowledge (void * buffer, std::size_t len, std::function <void(void *)> PerformBeforeAcknowledging);
   ssize_t SendLarge (const void * data, std::size_t len);
   ssize_t ReceiveLarge (void * buffer, std::size_t len);
   void CloseSocket ();
};

inline int SocketHandler::GetFileDescriptor ()
{
   return is_client_connected ? connected_socket_fd : socket_fd;
}

inline ssize_t SocketHandler::Send (const void * data, std::size_t len)
{
   return send(GetFileDescriptor(), data, len, 0);
}

inline ssize_t SocketHandler::Receive (void * buffer, std::size_t len)
{
   return recv(GetFileDescriptor(), buffer, len, 0);
}

inline void SocketHandler::CloseSocket ()
{
   close(GetFileDescriptor());
}
#endif
