// filename - videoSender.cpp

#include "SharedStructs.h"
#include "SignalHandling.h"
#include "SocketHandler.h"

#include <iostream>

#include <opencv2/opencv.hpp>

#include "process_metrics.h"

int SenderMainEntry (const char* videoFile)
{
   std::cout << "Video sender ..." << std::endl;

   cv::VideoCapture capturedVideo;
   capturedVideo.open(videoFile);
   if (!capturedVideo.isOpened())
   {
      std::cerr << "Unable to open video: " << videoFile << std::endl;
      return -1;
   }

   cv::namedWindow("Video Send", cv::WINDOW_AUTOSIZE);

   SocketHandler udpSocket(SOCK_DGRAM, PORT_TWO);
   udpSocket.ConnectToServer("127.0.0.1");

   SocketHandler tcpSocket(SOCK_STREAM, PORT_ONE);

   auto action = [] (int) { fputs("Exiting in response to SIGPIPE\n", stderr); exit(1); };
   RegisterHandlerForSignal(SIGPIPE, action);

   cv::Mat dispFrame;
   std::size_t frameSize;
   bool passOnce = false;
   while (true)
   {
      capturedVideo >> dispFrame;
      if (!passOnce)
      {
         passOnce = true;
         MatMetaData frameMetaData{dispFrame.rows, dispFrame.cols, dispFrame.type()};
         udpSocket.SendAndWait(&frameMetaData, sizeof(frameMetaData));
         tcpSocket.ConnectToServer("127.0.0.1");
         frameSize = dispFrame.total() * dispFrame.elemSize();
      }

      if (dispFrame.empty()) // end of file
      {
         break;
      }
      else
      {
         tcpSocket.SendLarge(dispFrame.data, frameSize);
         imshow("Video Send", dispFrame);
      }
      if (cv::waitKey(30) == 0) // wait for key press event
      {
         puts("Key press");
         break;
      }
   }

   return 0;
}

int main(int argv, char** argc)
{
   return SenderMainEntry(argc[1]);
}
